class TForgetable
{
public:
  virtual ~TForgetable();
};

class TNotifyingWeakPtrBase
{
public:
  virtual void NotifyPointeeGone();
};

template <typename T> class TWeakPtr
{
public:
  template <typename U> friend class TIPtr;
};

template <typename T> class TNotifyingWeakPtr : public TNotifyingWeakPtrBase
{
};

struct TDummy {};

class AObject
{
public:
  virtual ~AObject();
};

class TListable
{
public:
  virtual ~TListable();
};

class UList
{
public:
  virtual ~UList();

  friend class TListable;
};

template <typename T> class TList : public UList
{
};

template <typename T> class TBaseTypeListNode
{
public:
  virtual ~TBaseTypeListNode();
};

template <typename T> class TBaseTypeList : public TList< TBaseTypeListNode<T> >
{
};

template <typename T> class TPtr
{
};

template <typename T> class TLocal
{
};

class TString
{
};

class ACompound;

class TClassData
{
  typedef ACompound* (TNewFunc)();

  TBaseTypeList<TClassData*> mDerivedClasses;

  TNewFunc* mpNewFunc;
};

template < class Class > class AMember
{
public:
  virtual ~AMember();
};

class AAction
{
public:
  virtual ~AAction();
};

class AActionUpdate
{
public:
  virtual ~AActionUpdate();
};

template <class Class> class AList;

class AManeuver
{
public:
  typedef const void* TManeuverID;

  virtual ~AManeuver();

  AMember< AList<AAction> > mpUnDoActions, mpReDoActions;
};

class IManeuverHandler
{
  virtual AManeuver::TManeuverID OpenManeuverId() = 0;
};

class AFileRef;
class ARemoteableBang;
class TLomObjectRegistrar;

class ADocument : public IManeuverHandler
{
  static bool SCanRestoreDocument(
    AFileRef* pFileRef,
    TString* pDisplayName);

  virtual const TLomObjectRegistrar* LomObjectRegistrar() const;

  TLocal<ARemoteableBang> mpDirtyStateBang;
};

class ARemoteableEnum
{
public:
  virtual ~ARemoteableEnum();
};

class ADevice;

class TDeviceKeyMidiOwner
{
  TWeakPtr<ADevice> mpDevice;
};

class ADevice
{
public:
  virtual ~ADevice();
};

class ARemoteableNumericValue
{
public:
  virtual ~ARemoteableNumericValue();
};

class ARemoteableInt
{
public:
  virtual ~ARemoteableInt();
};

class ARemoteableBool
{
public:
  virtual ~ARemoteableBool();
};

class ALoop
{
public:
  virtual ~ALoop();

  class TSmallLoopLengthAllower
  {
    TWeakPtr<ALoop> mpLoop;
  };
};

class ARemoteableList
{
public:
  virtual ~ARemoteableList();
};

class AListBase
{
public:
  virtual ~AListBase();
};

template <class Class> class AList
{
public:
  virtual ~AList();
};

class ARelativePathElement
{
public:
  virtual ~ARelativePathElement();
};


class AFileRefSearchHint
{
public:
  virtual ~AFileRefSearchHint();

  AMember<AList<ARelativePathElement> > mpPathHint;
};

class AFileRef
{
public:
  virtual ~AFileRef();

  AMember<AFileRefSearchHint>           mpSearchHint;
};

class AClip;
class ASong;

class ASongTimeSelection
{
  TWeakPtr<ASong> mpSong;
};

class AClipTimeSelection
{
  TWeakPtr<AClip> mpClip;
};

class ATrack;

class AClip
{
public:
  virtual ~AClip();
  mutable TWeakPtr<ATrack> mpTrack;
};

class AControl;
class ATimeableValue;

class ATrack
{
  virtual ~ATrack();
};

class ASong
{
public:
  virtual ~ASong();
};

class AGroupDevice
{
public:
  virtual ~AGroupDevice();
};

class AGroupDeviceSectionController
{
  TWeakPtr<AGroupDevice> mpGroupDevice;
};

class ADrumGridLayout
{
  TWeakPtr<ARemoteableInt> mpSelectedRow;
};

class ARemoteableString
{
public:
  virtual ~ARemoteableString();
};

class ARemote;

class TRemoteableListenerEntry
{
  TNotifyingWeakPtr<TForgetable> mpListener;
};

typedef TRemoteableListenerEntry TRemoteableListenerList;

class ARemoteable
{
public:
  virtual ~ARemoteable();
  void AttachRemote(ARemote* pListener);
  virtual ARemoteableNumericValue* AsRemoteableValue();

  TRemoteableListenerList mListeners;
  friend class AActionUpdate;
};

class ARemoteableBang : public ARemoteable
{
public:
  virtual ~ARemoteableBang();
};

class ASelectionFrameKeyMidi;

class ARemoteableKeyMidi
{
  TWeakPtr<ASelectionFrameKeyMidi> mpSelectionFrameKeyMidi;

  TWeakPtr<ARemoteableString> mpMacroNameRemoteable;
};

class ADefaultSelectionFrame
{
  TWeakPtr<AControl> mpControl;
};

template <class T>
class TIPtr
{
};

class AStreamDevice
{
public:
  virtual ~AStreamDevice();
};

class ACompound
{
public:
  virtual ~ACompound();
  virtual void OnRemovedFromDocument(ADocument*) { }

  mutable ACompound* mpParentCompound;

  static TString SGetXmlStreamFormatVersion(AStreamDevice*);

  friend class AGlue;
  friend class ARemoteableList;
  friend class AListBase;
};

class AGlue
{
public:
  virtual ~AGlue();
};

class ARemote
{
public:
  virtual ~ARemote();
};

class TColor;

class AView
{
public:
  virtual ~AView();
};

class AControl
{
public:
  virtual ~AControl();

  TWeakPtr<AView> mpSelectionInfoTextView;
};

class AControlSelectionFrame
{
  TWeakPtr<ARemoteableBool> mpFrozen;
};

class AStringConverter;

class ASingleLineEditField
{
  TWeakPtr<AStringConverter> mpCurrentlyUsedConverter;
};

class ASelectionFrameKeyMidi
{
public:
  virtual ~ASelectionFrameKeyMidi();
};

class AUserFloatConverter
{
public:
  virtual ~AUserFloatConverter();
};


class TLomObjectRegistrar : public AObject
{
  ~TLomObjectRegistrar();
};


class AStringConverter
{
  virtual void StringToUserValue(TString String, double& Value, bool& IsValid);
};

class IValueRangeFixer;

class APointee : public ACompound
{
public:
  virtual ~APointee();
};

class IAutomationTarget
{
public:
  virtual ~IAutomationTarget() = 0;
  virtual AUserFloatConverter* UserFloatConverter() const = 0;
  virtual TIPtr<IValueRangeFixer> ValueRangeFixer() const = 0;
  virtual ARemoteableEnum* AutomationLedStateRemoteable(TDummy* pD = 0) = 0;
  virtual ATimeableValue* Timeable(TDummy* pD=0) = 0;
};

inline IAutomationTarget::~IAutomationTarget() {}

class AEnvelopeTarget : public APointee
  , public IAutomationTarget
{
public:
  virtual ~AEnvelopeTarget() = 0;
};

inline AEnvelopeTarget::~AEnvelopeTarget() {}

class ATimeableValue
{
  virtual ~ATimeableValue();
};

class APalette
{
public:
  virtual TClassData* ClassData() const;
};

class TSurfaceInfo
{
  TPtr<APalette>  pPalette;
};

class APixel
{
  TSurfaceInfo mSurfaceInfo;
};

class APixel32Rgba : public APixel
{
  virtual unsigned int ToNativeColor(TColor Col);
};

class APixel24Rgb
{
  virtual unsigned int ToNativeColor(TColor Col);
};

class AAlphaPixel
{
  virtual unsigned int ToNativeColor(TColor Col);
};

class TColor
{
};

inline unsigned int APixel32Rgba::ToNativeColor(TColor)
{
  return 0;
}
inline unsigned int APixel24Rgb::ToNativeColor(TColor)
{
  return 0;
}
inline unsigned int AAlphaPixel::ToNativeColor(TColor)
{
  return 0;
}
